from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.grammar_error import GrammarError


T = TypeVar("T", bound="GrammarErrors")


@_attrs_define
class GrammarErrors:
    """
    Attributes:
        edits (List['GrammarError']):
    """

    edits: List["GrammarError"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        edits = []
        for edits_item_data in self.edits:
            edits_item = edits_item_data.to_dict()
            edits.append(edits_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "edits": edits,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.grammar_error import GrammarError

        d = src_dict.copy()
        edits = []
        _edits = d.pop("edits")
        for edits_item_data in _edits:
            edits_item = GrammarError.from_dict(edits_item_data)

            edits.append(edits_item)

        grammar_errors = cls(
            edits=edits,
        )

        grammar_errors.additional_properties = d
        return grammar_errors

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.conversation import Conversation


T = TypeVar("T", bound="Message")


@_attrs_define
class Message:
    """
    Attributes:
        id (int):
        timestamp (datetime.datetime):
        message (str):
        is_system (bool):
        conversation_id (int):
        conversation (Conversation):
    """

    id: int
    timestamp: datetime.datetime
    message: str
    is_system: bool
    conversation_id: int
    conversation: "Conversation"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        timestamp = self.timestamp.isoformat()

        message = self.message

        is_system = self.is_system

        conversation_id = self.conversation_id

        conversation = self.conversation.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "timestamp": timestamp,
                "message": message,
                "is_system": is_system,
                "conversation_id": conversation_id,
                "conversation": conversation,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.conversation import Conversation

        d = src_dict.copy()
        id = d.pop("id")

        timestamp = isoparse(d.pop("timestamp"))

        message = d.pop("message")

        is_system = d.pop("is_system")

        conversation_id = d.pop("conversation_id")

        conversation = Conversation.from_dict(d.pop("conversation"))

        message = cls(
            id=id,
            timestamp=timestamp,
            message=message,
            is_system=is_system,
            conversation_id=conversation_id,
            conversation=conversation,
        )

        message.additional_properties = d
        return message

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="GrammarError")


@_attrs_define
class GrammarError:
    """
    Attributes:
        end (int):
        error_type (str):
        general_error_type (str):
        id (str):
        replacement (str):
        sentence (str):
        sentence_start (int):
        start (int):
        formatted_error_type (Union[None, Unset, str]):
    """

    end: int
    error_type: str
    general_error_type: str
    id: str
    replacement: str
    sentence: str
    sentence_start: int
    start: int
    formatted_error_type: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        end = self.end

        error_type = self.error_type

        general_error_type = self.general_error_type

        id = self.id

        replacement = self.replacement

        sentence = self.sentence

        sentence_start = self.sentence_start

        start = self.start

        formatted_error_type: Union[None, Unset, str]
        if isinstance(self.formatted_error_type, Unset):
            formatted_error_type = UNSET
        else:
            formatted_error_type = self.formatted_error_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "end": end,
                "error_type": error_type,
                "general_error_type": general_error_type,
                "id": id,
                "replacement": replacement,
                "sentence": sentence,
                "sentence_start": sentence_start,
                "start": start,
            }
        )
        if formatted_error_type is not UNSET:
            field_dict["formatted_error_type"] = formatted_error_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        end = d.pop("end")

        error_type = d.pop("error_type")

        general_error_type = d.pop("general_error_type")

        id = d.pop("id")

        replacement = d.pop("replacement")

        sentence = d.pop("sentence")

        sentence_start = d.pop("sentence_start")

        start = d.pop("start")

        def _parse_formatted_error_type(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        formatted_error_type = _parse_formatted_error_type(d.pop("formatted_error_type", UNSET))

        grammar_error = cls(
            end=end,
            error_type=error_type,
            general_error_type=general_error_type,
            id=id,
            replacement=replacement,
            sentence=sentence,
            sentence_start=sentence_start,
            start=start,
            formatted_error_type=formatted_error_type,
        )

        grammar_error.additional_properties = d
        return grammar_error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .body_login_for_access_token_users_token_post import BodyLoginForAccessTokenUsersTokenPost
from .conversation import Conversation
from .grammar_error import GrammarError
from .grammar_errors import GrammarErrors
from .http_validation_error import HTTPValidationError
from .message import Message
from .send_message_chat_send_message_post_response_send_message_chat_send_message_post import (
    SendMessageChatSendMessagePostResponseSendMessageChatSendMessagePost,
)
from .token import Token
from .user import User
from .validation_error import ValidationError

__all__ = (
    "BodyLoginForAccessTokenUsersTokenPost",
    "Conversation",
    "GrammarError",
    "GrammarErrors",
    "HTTPValidationError",
    "Message",
    "SendMessageChatSendMessagePostResponseSendMessageChatSendMessagePost",
    "Token",
    "User",
    "ValidationError",
)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.user import User


T = TypeVar("T", bound="Conversation")


@_attrs_define
class Conversation:
    """
    Attributes:
        id (int):
        creation_date (datetime.datetime):
        topic (Union[None, str]):
        active (Union[None, bool]):
        user_id (int):
        user (User):
    """

    id: int
    creation_date: datetime.datetime
    topic: Union[None, str]
    active: Union[None, bool]
    user_id: int
    user: "User"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        creation_date = self.creation_date.isoformat()

        topic: Union[None, str]
        topic = self.topic

        active: Union[None, bool]
        active = self.active

        user_id = self.user_id

        user = self.user.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "creation_date": creation_date,
                "topic": topic,
                "active": active,
                "user_id": user_id,
                "user": user,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user import User

        d = src_dict.copy()
        id = d.pop("id")

        creation_date = isoparse(d.pop("creation_date"))

        def _parse_topic(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        topic = _parse_topic(d.pop("topic"))

        def _parse_active(data: object) -> Union[None, bool]:
            if data is None:
                return data
            return cast(Union[None, bool], data)

        active = _parse_active(d.pop("active"))

        user_id = d.pop("user_id")

        user = User.from_dict(d.pop("user"))

        conversation = cls(
            id=id,
            creation_date=creation_date,
            topic=topic,
            active=active,
            user_id=user_id,
            user=user,
        )

        conversation.additional_properties = d
        return conversation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """
    Attributes:
        id (int):
        creation_date (datetime.datetime):
        username (str):
        cefr_level (str):
        disabled (bool):
    """

    id: int
    creation_date: datetime.datetime
    username: str
    cefr_level: str
    disabled: bool
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        creation_date = self.creation_date.isoformat()

        username = self.username

        cefr_level = self.cefr_level

        disabled = self.disabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "creation_date": creation_date,
                "username": username,
                "cefr_level": cefr_level,
                "disabled": disabled,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        creation_date = isoparse(d.pop("creation_date"))

        username = d.pop("username")

        cefr_level = d.pop("cefr_level")

        disabled = d.pop("disabled")

        user = cls(
            id=id,
            creation_date=creation_date,
            username=username,
            cefr_level=cefr_level,
            disabled=disabled,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
